//------------------------------------------------------------------------------
// File: Auxiliary.cpp
//
// Desc: Useful auxiliaries for the Win32 Application.
//
// Copyright (c) 2000 - 2005, L544 Technology.  All rights reserved.
//------------------------------------------------------------------------------

#include "stdafx.h"

#ifdef _DEBUG

#include <stdio.h>

void TRACE(LPCTSTR lpszFormat, ...)
{

    va_list args ;
    va_start(args, lpszFormat) ;

	int nBuf ;
    TCHAR szBuffer[1024] ;  // Large buffer for very long filenames (like with HTTP)

	nBuf = vsprintf(szBuffer, lpszFormat, args) ;

	// was there an error? was the expanded string too long?
	assert(nBuf >= 0) ;

	OutputDebugString(szBuffer) ;
//	MessageBox(NULL, szBuffer, "DirectShow", MB_OK) ;

    va_end(args) ;
}

#else

void TRACE(LPCTSTR lpszFormat, ...)
{
	// TODO: Place code here.
}

#endif

// An application can advertise the existence of its filter graph
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//

#ifdef REGISTER_FILTERGRAPH

HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
    IMoniker * pMoniker;
    IRunningObjectTable *pROT;
    if (FAILED(GetRunningObjectTable(0, &pROT))) {
        return E_FAIL;
    }
    WCHAR wsz[128];
    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
    HRESULT hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) {
        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();
    return hr;
}




void RemoveGraphFromRot(DWORD pdwRegister)
{
    IRunningObjectTable *pROT;
    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) {
        pROT->Revoke(pdwRegister);
        pROT->Release();
    }
}

#endif


HRESULT ConnectFilters(IGraphBuilder* pGB, IBaseFilter* pUpFilter, IBaseFilter* pDownFilter, int nMAX)
{
	HRESULT hr;
	
	// Get the first output pin of the new source filter. Audio sources 
	// typically have only one output pin, so for most audio cases finding 
	// any output pin is sufficient.
	
	IEnumPins *pUpEnumPins = NULL, *pDownEnumPins = NULL;
	IPin *ppinOut = NULL, *pDownstreamPin;
	IPin *ppinIn = NULL, *pUpstreamPin;
	PIN_DIRECTION direction;
	bool bIsAnySuccess = false;
	
	JIF(pUpFilter->EnumPins(&pUpEnumPins));
	
	while (pUpEnumPins->Next(1, &ppinOut, 0) == S_OK)
	{
		JIF(ppinOut->QueryDirection(&direction));
		
		if (direction == PINDIR_OUTPUT)
		{
			ppinOut->ConnectedTo(&pDownstreamPin);
			
			if (pDownstreamPin == NULL)
			{
				// Downstream filter: pin enumeration
				JIF(pDownFilter->EnumPins(&pDownEnumPins));
				
				while (pDownEnumPins->Next(1, &ppinIn, 0) == S_OK)
				{
					JIF(ppinIn->QueryDirection(&direction));
					
					if (direction == PINDIR_INPUT)
					{
						ppinIn->ConnectedTo(&pUpstreamPin);
						
						if (pUpstreamPin == NULL)
						{
							//
							// We have two pins. Connect it
							//
							
							//      
							// ߻ϴ   ´!
							if (nMAX-- > 0)
							{
								hr = pGB->Connect(ppinOut, ppinIn);
							}
							
							if (hr == S_OK)
								bIsAnySuccess |= true;
							else if (hr == VFW_S_PARTIAL_RENDER)
								bIsAnySuccess |= false;
							else if (hr == E_ABORT)
								bIsAnySuccess |= false;
							else if (hr == E_POINTER)
								bIsAnySuccess |= false;
							else if (hr == VFW_E_CANNOT_CONNECT)
								bIsAnySuccess |= false;
							else if (hr == VFW_E_NOT_IN_GRAPH)
								bIsAnySuccess |= false;
						}
						SAFE_RELEASE(pUpstreamPin);
					}
					SAFE_RELEASE(ppinIn);
				}
				SAFE_RELEASE(pDownEnumPins);
			}
			// end of downstream
			SAFE_RELEASE(pDownstreamPin);
		}
		SAFE_RELEASE(ppinOut);
	}
	SAFE_RELEASE(pUpEnumPins);
	// end of upstream
	
	return (bIsAnySuccess == false ? E_FAIL : S_OK);
}



