//------------------------------------------------------------------------------
// File: JYPlayer.cpp
//
// Desc: Defines the entry point for the application.
//
// Copyright (c) 2000 - 2005, L544 Technology.  All rights reserved.
//------------------------------------------------------------------------------

#include "stdafx.h"
#include "JYPlayerApp.h"
#include "resource.h"
#include <commctrl.h>

#include <initguid.h>
#include "myid.h"


//------------------------------------------------------------------------------
// Global data
//------------------------------------------------------------------------------

const PTSTR APPNAME = TEXT("JYPlayer") ;
CJYPlayerApp g_App ;


//------------------------------------------------------------------------------
// Name: WinMain()
// Desc: This method is the standard Windows program entry point.
//------------------------------------------------------------------------------

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	g_App.SetAppValues(hInstance, APPNAME, IDS_APP_TITLE) ;

	// Perform application initialization:
	if (! g_App.InitInstance(nCmdShow) ) 
	{
		return (FALSE) ;
	}

	HACCEL hAccelTable ;
	hAccelTable = LoadAccelerators(hInstance, g_App.GetAppName()) ;

	// Main message loop:
	MSG msg ;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (! TranslateAccelerator(msg.hwnd, hAccelTable, &msg) )
		{
			TranslateMessage(&msg) ;
			DispatchMessage(&msg) ;
		}
	}

	return (msg.wParam) ;
}



//------------------------------------------------------------------------------
// Name: WndProc()
// Desc: This method is our main window procedure.  It parses the messages
//       and farms out the work to other procedures.
// Note: It's a static function. We should use g_App to access members.
//------------------------------------------------------------------------------

LRESULT CJYPlayerApp::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HDC  hDC ;
	static PAINTSTRUCT ps ;

	switch (message) 
	{
	case WM_PAINT:
		hDC = BeginPaint(hWnd, &ps) ;
		assert(hDC) ;
		g_App.OnDraw(hDC) ;
		EndPaint(hWnd, &ps) ;
		break ;
    
	case WM_KEYUP:
		g_App.KeyProc(wParam, lParam) ;
		break ;

	case WM_HSCROLL:	
	g_App.MoveTrackBar((HWND)lParam, LOWORD(wParam));
	break;

	case WM_TIMER:
	g_App.OnTimer();
	break;
    
	case WM_COMMAND:
		g_App.MenuProc(hWnd, wParam, lParam) ;
		break ;
    
	case WM_NOTIFY:
		return g_App.ToolTipProc(hWnd, message, wParam, lParam) ;
		break ;

	case WM_GRAPHNOTIFY:
		g_App.HandleGraphEvent(lParam) ;
		break ;

	case WM_DESTROY:
		PostQuitMessage(0) ;
		break;

	case WM_SIZE:
		// we do this to cause the toolbar to resize correctly
		SendMessage(g_App.m_hwndToolBar, WM_SIZE, wParam, lParam) ;
		g_App.OnSize() ;
		return (DefWindowProc(hWnd, message, wParam, lParam)) ;
		break ;
    
	default:
		return (DefWindowProc(hWnd, message, wParam, lParam)) ;
	}

	return (0) ; // let Windows know we handled the message
}


//------------------------------------------------------------------------------
// Name: ToolTipProc()
// Desc: This method is a MessageProc for our toolbar tooltips.
//       This checks whether the WM_NOTIFY message is really a tooltip.
//       If it is, we compare it to the buttons on the toolbar and send the right text back.
//       Note that older code would use TTN_NEEDTEXT instead of TTN_GETDISPINFO.
//------------------------------------------------------------------------------

LRESULT CJYPlayerApp::ToolTipProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	LPNMTTDISPINFO TText ;
	TText = (LPNMTTDISPINFO) lParam ;

	if (TTN_GETDISPINFO == TText->hdr.code)
	{
		switch (TText->hdr.idFrom)
		{
			case IDM_PLAYBACK_PLAY:
				TText->lpszText = TEXT("Play") ;
				break ;

			case IDM_PLAYBACK_PAUSE:
				TText->lpszText = TEXT("Pause") ;
				break ;

			case IDM_PLAYBACK_STOP:
				TText->lpszText = TEXT("Stop") ;
				break ;

			case IDM_PLAYBACK_REWIND:
				TText->lpszText = TEXT("Rewind") ;
				break ;

			case IDM_PLAYBACK_FASTFORWARD:
				TText->lpszText = TEXT("FastForward") ;
				break ;

			case IDM_PLAYBACK_PREVIOUS:
				TText->lpszText = TEXT("Previous") ;
				break ;

			case IDM_PLAYBACK_NEXT:
				TText->lpszText = TEXT("Next") ;
				break ;

			// TODO: Place code here.
		}
		return (0) ;
	}
	else
		return (DefWindowProc(hWnd, message, wParam, lParam)) ; // it wasn't a tooltip message
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::MenuProc()
// Desc: This method handles all of the menu messages for our application.  It is
//       passed these messages by the main message proc.
//------------------------------------------------------------------------------

LRESULT CJYPlayerApp::MenuProc(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	HMENU hMenu = GetMenu(hWnd) ;

	// Parse the menu selections:
	switch (LOWORD(wParam)) 
	{
	case IDM_EXIT:
		DestroyWindow(m_hWnd) ;
		break ;

	case IDM_FILE_OPEN:
		OnFileOpen() ;
		break ;

	case IDM_FILE_CLOSE:
		OnFileClose() ;
		break ;

    case IDM_PLAYBACK_PLAY:
		OnPlayClip() ;
		break ;

    case IDM_PLAYBACK_PAUSE:
		OnPauseClip() ;
		break ;

    case IDM_PLAYBACK_STOP:
		OnStopClip() ;
		break ;

    case IDM_PLAYBACK_REWIND:
		OnRewindClip() ;
		break ;

    case IDM_PLAYBACK_FASTFORWARD:
		OnFastForwardClip() ;
		break ;

    case IDM_PLAYBACK_PREVIOUS:
		OnPreviousClip() ;
		break ;

	case IDM_PLAYBACK_SCREEN:
		CaptureScreen();
		break;

    case IDM_PLAYBACK_NEXT:
		OnNextClip() ;
		break ;
	
	case IDM_VOLUME_DOWN:
		OnVolume(false);
		break;
	case IDM_VOLUME_UP:
		OnVolume(true);
		break;

	case IDM_BALANCE_LEFT:
		OnBalance(false);
		break;

	case IDM_BALANCE_RIGHT:
		OnBalance(true);
		break;

	case ID_Capture:
		CaptureScreen();
		break;

	case ID_URLOPEN:
		OnURLOpen();
		OnPlayClip();
		break;
	
	case ID_Capture_View:
		CaptureView();
		break;
	case ID_BIT:
		OnFileOpenBit();
		OnPlayClip();
		break;

	case ID_FILTER:
		OnFilter();
		OnPlayClip();
		break;
	// TODO: Place code here.

	default:
		break ;
	}

	return (0);
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::KeyProc()
// Desc: This method will process all key presses sent to our application window.
//       At present it passes all of the keys along to the DvdCore but this is where
//       you would implement shortcut keys, etc.
//------------------------------------------------------------------------------

LRESULT CJYPlayerApp::KeyProc(WPARAM wParam, LPARAM lParam)
{
	switch (wParam)
	{
	case VK_ESCAPE:		// exit full screen
		DestroyWindow(m_hWnd);
		break ;

	case VK_RETURN:		// activate the currently selected button
		break ;

	case VK_LEFT:		// select the left button
		OnBalance(false);
		break ;

	case VK_RIGHT:		// select the right button
		OnBalance(true);
		break ;

	case VK_DOWN:		// select the lower button
		OnVolume(false);
		break ;

	case VK_UP:			// select the upper button
		OnVolume(true);
		break ;

	// TODO: Place code here.
	case 'F':
		ToggleFullScreen();
		break ;
	case VK_SNAPSHOT:
		CaptureScreen();

	}
	return (DefWindowProc(m_hWnd, WM_KEYUP, wParam, lParam)) ;
}
