//------------------------------------------------------------------------------
// File: JYPlayer.cpp
//
// Desc: Implementation of the CJYPlayerApp class.
//
// Copyright (c) 2000 - 2005, L544 Technology.  All rights reserved.
//------------------------------------------------------------------------------

#include "stdafx.h"
#include "JYPlayerApp.h"
#include "RenderURLDlg.h"
#include "resource.h"

#include <commctrl.h>
#pragma comment(lib, TEXT("comctl32.lib"))

#include <commdlg.h>
#include <math.h>


#include "myid.h"
#include <qedit.h>
#include <initguid.h>  // CLSID
#include "fInplaceTransform.h"


//------------------------------------------------------------------------------
// Type Definitions
//------------------------------------------------------------------------------

typedef UINT (CALLBACK* PFNDLL_STES)(UINT);

// File filter for OpenFile dialog
#define FILE_FILTER_TEXT \
	TEXT("Video Files (AVI,ASF,MPEG,Indeo,QuickTime)\0*.avi;*.asf;*.wmv;*.mpg;*.mpe*;*.mp1*;*.mp2*;*.mpv*;*.ivf;*.qt\0")\
	TEXT("Image files (BMP,JPG,GIF)\0*.bmp;*.jpg;*.gif\0")\
    TEXT("All Files (*.*)\0*.*;\0\0")

#define FILTER_TEXT \
	TEXT("Image files (BMP,JPG,GIF)\0*.bmp;*.jpg;*.gif\0")\
	TEXT("Video Files (AVI,ASF,MPEG,Indeo,QuickTime)\0*.avi;*.asf;*.wmv;*.mpg;*.mpe*;*.mp1*;*.mp2*;*.mpv*;*.ivf;*.qt\0")\
	TEXT("All Files (*.*)\0*.*;\0\0")

// Begin default media search at root directory
#define DEFAULT_MEDIA_PATH  TEXT("C:\\Documents and Settings\\admin\\My Documents\\My Pictures\\")

#define DIB_HEADER_MARKER ((WORD)('M'<<8) | 'B')

//------------------------------------------------------------------------------
// Name: CJYPlayerApp::CJYPlayerApp()
// Desc: This is the constructor for CJYPlayerApp.  It sets default values and disables 
//       power saving.
//------------------------------------------------------------------------------

CJYPlayerApp::CJYPlayerApp() : m_hInstance(NULL), 
               m_hWnd(NULL), 
               m_hwndToolBar(NULL),
#ifdef REGISTER_FILTERGRAH
			   m_dwGraphRegister(0),
#endif
			   m_bFullScreenOn(false), m_lOrgStyle(0), m_lOrgStyleEx(0),
			   m_fVolumePos(0.5), m_fBalancePos(0),
			   m_pGB(NULL),
			   m_pSG(NULL)
	
{
    ZeroMemory(m_szAppTitle, sizeof(m_szAppTitle)) ;
    ZeroMemory(m_szAppName, sizeof(m_szAppName)) ;
	ZeroMemory(m_szFileName, sizeof(m_szFileName)) ;
	ZeroMemory(&m_rectOrgVideo, sizeof(m_rectOrgVideo)) ;

    // Initialize COM
    if (FAILED(CoInitialize(NULL)))
    {
        TRACE(TEXT("CoInitialize Failed!\r\n")) ;
        exit(1) ;
    }

    // notify the power manager that the display is in use so it won't go to sleep
	// SetThreadExecutionState() is a Win98/Win2k API.
	// The following code lets it fail gracefully on Win95

	PFNDLL_STES pfn ;
	pfn = (PFNDLL_STES) GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")),
		"SetThreadExecutionState") ;
	if (pfn)
	{
		pfn(ES_CONTINUOUS | ES_DISPLAY_REQUIRED) ;
	}
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::~CJYPlayerApp()
// Desc: This is the destructor for CJYPlayerApp.
//------------------------------------------------------------------------------

CJYPlayerApp::~CJYPlayerApp()
{
	// ͱ׷Ŵ  
	if (m_pGB != NULL)
	{
		OnFileClose() ;
	}

    // Finished with COM
    CoUninitialize() ;

	// notify the power manager that the display is no longer in use and it can go to sleep
	// SetThreadExecutionState() is a Win98/Win2k API.
	// The following code lets it fail gracefully on Win95

	PFNDLL_STES pfn;
	pfn = (PFNDLL_STES) GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")),
		"SetThreadExecutionState") ;
	if (pfn)
	{
		pfn(ES_CONTINUOUS) ;
	}
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::SetAppValues()
// Desc: This method sets the basic application values like szAppName and hInstance
//------------------------------------------------------------------------------

void CJYPlayerApp::SetAppValues(HINSTANCE hInst, PTSTR szAppName, int iAppTitleResId)
{
	// The Windows stuff
	m_hInstance = hInst ;
	lstrcpy(m_szAppName, szAppName) ;
	LoadString(m_hInstance, IDS_APP_TITLE, m_szAppTitle, 100) ;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::InitInstance()
// Desc: This method registers and creates our window and toolbar.
//------------------------------------------------------------------------------

bool CJYPlayerApp::InitInstance(int nCmdShow)
{
	// Win32 will always set hPrevInstance to NULL, So lets check
	// things a little closer. This is because we only want a single
	// version of this JYPlayerApp to run at a time
	m_hWnd = FindWindow (m_szAppName, m_szAppTitle) ;
	if (m_hWnd) 
	{
		// We found another instance of ourself. Lets use that one:
		if (IsIconic(m_hWnd)) 
		{
			ShowWindow(m_hWnd, SW_RESTORE) ;
		}
		SetForegroundWindow(m_hWnd) ;
        
		// If this JYPlayerApp actually had any methodality, we would
		// also want to communicate any action that our 'twin'
		// should now perform based on how the user tried to
		// execute us.
		return false;
	}
	 
		
	// Register the JYPlayerApp main window class
	WNDCLASSEX  wc ;
	wc.cbSize        = sizeof(wc) ;
	wc.style         = CS_HREDRAW | CS_VREDRAW ;
	wc.lpfnWndProc   = (WNDPROC) WndProc ;
	wc.cbClsExtra    = 0 ;
	wc.cbWndExtra    = 0 ;
	wc.hInstance     = m_hInstance ;
	wc.hIcon         = LoadIcon(m_hInstance, MAKEINTRESOURCE(IDI_APPICON)) ;
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW) ;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+2) ;
	wc.lpszMenuName  = (LPCSTR)IDR_APPMENU ;
	wc.lpszClassName = m_szAppName ;
	wc.hIconSm       = LoadIcon(m_hInstance, MAKEINTRESOURCE(IDI_SMALL)) ;

	if (0 == RegisterClassEx(&wc))
	{
		TRACE("ERROR: RegisterClassEx() for JYPlayerApp class failed (Error %ld)\n", GetLastError()) ;
		return false ;
	}

	// Determine where to put the Application Window
	RECT rDesktop ;
	SystemParametersInfo(SPI_GETWORKAREA, NULL, &rDesktop, NULL) ;

	// Create an instance of the window we just registered
	// locate it at the center of the screen
	m_hWnd = CreateWindowEx(0, m_szAppName, m_szAppTitle, WS_OVERLAPPEDWINDOW,
		(rDesktop.right - rDesktop.left) / 2 - 150,
		(rDesktop.bottom - rDesktop.top) / 2 - 80,
		500, 300, NULL, NULL, m_hInstance, NULL) ;
	if (! m_hWnd)
	{
		TRACE("ERROR: CreateWindowEx() failed (Error %ld)\n", GetLastError()) ;
		return false ;
	}

	// We now create the toolbar
	INITCOMMONCONTROLSEX cc ;
	cc.dwSize = sizeof(INITCOMMONCONTROLSEX) ;
	cc.dwICC = ICC_BAR_CLASSES ; // register only the toolbar control
	InitCommonControlsEx(&cc) ;

	TBBUTTON tbb[] = 
	{
		0, IDM_PLAYBACK_PREVIOUS, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		1, IDM_PLAYBACK_REWIND, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		2, IDM_PLAYBACK_PLAY, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		3, IDM_PLAYBACK_PAUSE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		4, IDM_PLAYBACK_STOP, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		5, IDM_PLAYBACK_FASTFORWARD, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		6, IDM_PLAYBACK_NEXT, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		7, IDM_PLAYBACK_SCREEN, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0, 0,
		// TODO: Place code here.
	} ;

	m_hwndToolBar = CreateToolbarEx(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | CCS_TOP
		| TBSTYLE_TOOLTIPS, 1, 10, m_hInstance, IDR_APPTOOLBAR, tbb, sizeof(tbb)/sizeof(tbb[0]), 0, 0, 0, 0,
		sizeof(TBBUTTON))  ;
	if (! m_hwndToolBar) 
	{
		TRACE("ERROR: CreateToolbarEx() failed (Error %ld)\n", GetLastError()) ;
		return false ;
	}

	FileSeekTrackBar = CreateWindow(TRACKBAR_CLASS, 0, WS_CHILD | WS_VISIBLE | SBS_HORZ, 450, 0, 150, 20, m_hWnd, (HMENU)ID_FILESEEK, GetModuleHandle(0), 0);
//	SoundSeekTrackBar = CreateWindow(TRACKBAR_CLASS, 0, WS_CHILD | WS_VISIBLE | SBS_HORZ, 1250, 0, 200, 20, m_hWnd, (HMENU)ID_FILESEEK, GetModuleHandle(0), 0);

	SetParent(Filename = CreateWindow(TEXT("static"), TEXT(""), WS_CHILD | WS_VISIBLE, 200, 0, 150, 20, m_hWnd, (HMENU)-1, GetModuleHandle(0), 0), m_hwndToolBar);
	SetParent(NowTimer = CreateWindow(TEXT("static"), TEXT(""), WS_CHILD | WS_VISIBLE, 350, 0, 60, 20, m_hWnd, (HMENU)-1, GetModuleHandle(0), 0), m_hwndToolBar);
	SetParent(TotalTimer = CreateWindow(TEXT("static"), TEXT(""), WS_CHILD | WS_VISIBLE, 400, 0, 60, 20, m_hWnd, (HMENU)-1, GetModuleHandle(0), 0), m_hwndToolBar);
	SetParent(FileSeekTrackBar, m_hwndToolBar);  

//	SetParent(SoundSeekTrackBar, m_hwndToolBar);
	
	FileTrackPos =0;
	Die=TRUE;
	TimerCheck=TRUE;
	TimerCheck2=TRUE;

	// and finally, we make the window visible
	ShowWindow(m_hWnd, nCmdShow) ;
	UpdateWindow(m_hWnd) ;

	return true ;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::OnDraw()
// Desc: This method draws something on the screen
//------------------------------------------------------------------------------

void CJYPlayerApp::OnDraw(HDC hDC)
{
	// TODO: Place code here.
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::OnFileOpen()
// Desc: This method open a file dialog and render the selected file.
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::OnFileOpen()
{
	HRESULT hr ;

	// ͱ׷Ŵ Ͽ ͱ׷  !
	if (m_pGB != NULL)
	{
		JIF(OnFileClose()) ;
	}

	// ̵  ҷ
	// Open File Dialog ...
    static OPENFILENAME ofn={0} ;
    static BOOL bSetInitialDir = FALSE ;

    ofn.lStructSize       = sizeof(OPENFILENAME) ;
    ofn.hwndOwner         = m_hWnd ;
    ofn.lpstrFilter       = FILE_FILTER_TEXT ;
    ofn.lpstrCustomFilter = NULL ;
    ofn.nFilterIndex      = 1 ;
    ofn.lpstrFile         = m_szFileName ;
    ofn.nMaxFile          = MAX_PATH ;
    ofn.lpstrTitle        = TEXT("Open Media File...\0") ;
    ofn.lpstrFileTitle    = FileTitle ;
	ofn.nMaxFileTitle	  = MAX_PATH;
    ofn.lpstrDefExt       = TEXT("*\0") ;
    ofn.Flags             = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST ;

    // Remember the path of the first selected file
    if (bSetInitialDir == FALSE)
    {
        ofn.lpstrInitialDir = DEFAULT_MEDIA_PATH ;
        bSetInitialDir = TRUE ;
    }
    else 
        ofn.lpstrInitialDir = NULL ;

	// Retrieve the name of the selected file
	WCHAR wFileName[MAX_PATH] ;

	if (GetOpenFileName((LPOPENFILENAME)&ofn))
	{
#ifndef UNICODE
		MultiByteToWideChar(CP_ACP, 0, m_szFileName, -1, wFileName, MAX_PATH) ;
#else
		lstrcpy(wFileName, m_szFileName);
#endif
	}
	else
		return E_ABORT ;

    // Clear open dialog remnants before calling RenderFile()
	SetWindowText(m_hWnd, m_szFileName) ;
    UpdateWindow(m_hWnd) ;

	// ͱ׷Ŵ  (  )
    // Get the interface for DirectShow's GraphBuilder
    JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGB)) ;

	// Add a Filter ... Put a filter in the graph

/*
	IBaseFilter *pFilterOverlayMixer;
    LIF(CoCreateInstance(CLSID_L544_OverlayMixer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pFilterOverlayMixer)) ;
	LIF(m_pGB->AddFilter(pFilterOverlayMixer, L"Overlay Mixer"));

	IBaseFilter *pFilterInfinitePinTeeFilter;
    LIF(CoCreateInstance(CLSID_L544_InfinitePinTeeFilter, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pFilterInfinitePinTeeFilter)) ;
	LIF(m_pGB->AddFilter(pFilterInfinitePinTeeFilter, L"Infinite Pin Tee Filter"));
	
	

	// Release filter objects ...
	SAFE_RELEASE(pFilterOverlayMixer);
	SAFE_RELEASE(pFilterInfinitePinTeeFilter);
*/

// --------------------------ĸ ----------------------------
	LIF(CoCreateInstance(CLSID_L544_SampleGrabber, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&m_pSG)) ;
	LIF(m_pGB->AddFilter(m_pSG, L"SampleGrabber"));
	
	AM_MEDIA_TYPE mt;
	ZeroMemory(&mt, sizeof(mt));
	mt.majortype = MEDIATYPE_Video;
	mt.subtype = MEDIASUBTYPE_RGB24;
	
	CComQIPtr<ISampleGrabber> pSampleGrabber(m_pSG);
	if (pSampleGrabber != NULL)
	{
		LIF(pSampleGrabber->SetMediaType(&mt));
		LIF(pSampleGrabber->SetBufferSamples(TRUE));
	}


	// Render File ...
	// Have the graph construct its the appropriate graph automatically
	JIF(m_pGB->RenderFile(wFileName, NULL)) ;

	// Query for video interfaces
	IVideoWindow * pVW ;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW)) ;
	if (pVW != NULL)
	{
		LIF(pVW->put_Owner((OAHWND)m_hWnd)) ;
		LIF(pVW->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN)) ;

		// resizing
		LIF(OnSize());
	}
	// Release a interface
	
	SAFE_RELEASE(pVW) ;


	// Get the video size
	IBasicVideo * pBV ;
	JIF(m_pGB->QueryInterface(IID_IBasicVideo, (void **)&pBV)) ;
	if (pBV != NULL)
	{
		long lWidth, lHeight ;
		hr = pBV->GetVideoSize(&lWidth, &lHeight) ;
		if (SUCCEEDED(hr))
		{
			// Track the movement of the container window and resize as needed
			RECT rcToolbar ;
			GetWindowRect(m_hwndToolBar, &rcToolbar) ;
			int nToolbarHeight = rcToolbar.bottom - rcToolbar.top ;

			SetWindowPos(m_hWnd, HWND_NOTOPMOST, 0, 0,
				lWidth, lHeight + nToolbarHeight,
				SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE) ;
		}
	}
	// Release a interface
	SAFE_RELEASE(pBV) ;


	IMediaSeeking * pVWs;
		
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pVWs)) ;
	if (pVWs != NULL)
	{
		LONGLONG Duration;

		pVWs->GetDuration(&Duration);
		totaltime = Duration / ONE_SECOND;
		
		SendMessage(FileSeekTrackBar, TBM_SETRANGE, FALSE, MAKELPARAM(0, totaltime));
		
		nowm= (long)(Duration / ONE_SECOND) / 60;
		nowh = nowm / 60;
		nows = (long)(Duration / ONE_SECOND) % 60;
		StaticSetTime(TotalTimer);

	}
	SAFE_RELEASE(pVWs) ;


#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(m_pGB, &m_dwGraphRegister) ;
    if (FAILED(hr))
    {
        TRACE(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr) ;
        m_dwGraphRegister = 0 ;
    }
#endif

	return S_OK ;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::OnFileClose()
// Desc: This method release the filter graph manager.
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::OnFileClose()
{
	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		return E_ABORT ;

    HRESULT hr ;
 
	// Query for media control interfaces
	IMediaControl * pMC ;
	JIF(m_pGB->QueryInterface(IID_IMediaControl, (void **)&pMC)) ;
	if (pMC != NULL)
	{
		// Stop the graph to stop the media file
		LIF(pMC->Stop()) ;
	}
	// Release a interface
	SAFE_RELEASE(pMC) ;


	// Query for media event interfaces
	IMediaEventEx * pME;
	JIF(m_pGB->QueryInterface(IID_IMediaEventEx, (void **)&pME));
	if (pME != NULL)
	{
		// Relinquish ownership (IMPORTANT!)
		LIF(pME->SetNotifyWindow(NULL, WM_GRAPHNOTIFY, 0));
	}
	// Release a interface
	SAFE_RELEASE(pME);


	// Query for video interfaces
	IVideoWindow * pVW;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW));
	if (pVW != NULL)
	{
		// Relinquish ownership (IMPORTANT!) after hiding video window
		LIF(pVW->put_Visible(OAFALSE));
		LIF(pVW->put_Owner(NULL));
	}
	// Release a interface
	SAFE_RELEASE(pVW);


#ifdef REGISTER_FILTERGRAPH
    if (m_dwGraphRegister)
    {
        RemoveGraphFromRot(m_dwGraphRegister) ;
        m_dwGraphRegister = 0 ;
    }
#endif

	// ͱ׷Ŵ Ͽ ͱ׷  !
	// Release Filter Graph Manager
	SAFE_RELEASE(m_pGB) ;
	SAFE_RELEASE(m_pSG) ;
	// ⺻  ŸƲ 
	SetWindowText(m_hWnd, m_szAppTitle) ;

	return S_OK ;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::OnXXXClip()
// Desc: These methods manipulate playback features.
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::OnPlayClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Let's get ready to rumble!
	ShowWindow(m_hWnd, SW_SHOW) ;
	UpdateWindow(m_hWnd) ;
	SetForegroundWindow(m_hWnd) ;
	SetFocus(m_hWnd) ;

	// Query for media control interfaces
	IMediaControl * pMC ;
	JIF(m_pGB->QueryInterface(IID_IMediaControl, (void **)&pMC)) ;
	if (pMC != NULL)
	{
		// Run the graph to play the media file		
		SetTimer(m_hWnd, 1, 1000, 0);
		
		if(TimerCheck == TRUE && TimerCheck2 == TRUE)
		{
			nowh= 0;
			nowm = 0;
			nows = 0;
			SetWindowText(Filename, FileTitle);
		}
		LIF(pMC->Run()) ;
	}

	SAFE_RELEASE(pMC) ;

	// give a focus to the Video window, again!
	SetFocus(m_hWnd) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnPauseClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media control interfaces
	IMediaControl * pMC ;
	JIF(m_pGB->QueryInterface(IID_IMediaControl, (void **)&pMC)) ;
	if (pMC != NULL)
	{
		// Pause the graph to pause the media file
		LIF(pMC->Pause()) ;
	}

	if(!TimerCheck)
	{
		KillTimer(m_hWnd, 1);
		TimerCheck = TRUE;
		TimerCheck2 = FALSE;
	}
	else
	{
		SetTimer(m_hWnd, 1, 1000, 0);
	}

	// Release a interface
	SAFE_RELEASE(pMC) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnStopClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media control interfaces
	IMediaControl * pMC ;
	JIF(m_pGB->QueryInterface(IID_IMediaControl, (void **)&pMC)) ;
	if (pMC != NULL)
	{
		// Stop the graph to stop the media file
		LIF(pMC->Stop()) ;
	}

	// Query for media seeking interfaces
	IMediaSeeking * pMS ;
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pMS)) ;
	if (pMS != NULL)
	{
		// Go to the previous of the graph, usually the very first.
		LONGLONG pos = 0 ;
		LIF(pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning)) ;

		// Display the first frame to indicate the reset condition
		LIF(pMC->Pause()) ;
	}

	KillTimer(m_hWnd, 1);
	TimerCheck2 = TRUE;

	// Release a interface
	SAFE_RELEASE(pMS) ;
	// Release a interface
	SAFE_RELEASE(pMC) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnRewindClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media seeking interfaces
	IMediaSeeking * pMS ;
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pMS)) ;
	if (pMS != NULL)
	{
		// Rewind the graph to rewind the media file
		LONGLONG pos;
		LIF(pMS->GetPositions(&pos, NULL)) ;

		pos -= ONE_SECOND;
		FileTrackPos = (long)(pos / ONE_SECOND);
	
		nowm= (long)(pos / ONE_SECOND) / 60;
		nowh = nowm / 60;
		nows = (long)(pos / ONE_SECOND) % 60;

		LIF(pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning)) ;
	}
	// Release a interface
	SAFE_RELEASE(pMS) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnFastForwardClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media seeking interfaces
	IMediaSeeking * pMS ;
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pMS)) ;
	if (pMS != NULL)
	{
		// Fastforward the graph to fastforward the media file
		LONGLONG pos;
		LIF(pMS->GetPositions(&pos, NULL)) ;

		pos += ONE_SECOND;

		FileTrackPos = (long)(pos / ONE_SECOND);

		nowm= (long)(pos / ONE_SECOND) / 60;
		nowh = nowm / 60;
		nows = (long)(pos / ONE_SECOND) % 60;

		LIF(pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning)) ;
	}
	// Release a interface
	SAFE_RELEASE(pMS) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnPreviousClip(LONGLONG pos)
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media seeking interfaces
	IMediaSeeking * pMS ;
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pMS)) ;
	if (pMS != NULL)
	{
		pos *= ONE_SECOND;
		pos += ONE_SECOND;
		// Go to the previous of the graph, usually the very first.

		LIF(pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning)) ;
	}
	// Release a interface
	SAFE_RELEASE(pMS) ;

	return hr ;
}

HRESULT CJYPlayerApp::OnNextClip()
{
	HRESULT hr ;

	// ͱ׷Ŵ   
	if (m_pGB == NULL)
		JIF(OnFileOpen()) ;

	// Query for media seeking interfaces
	IMediaSeeking * pMS ;
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pMS)) ;
	if (pMS != NULL)
	{
		// Go to the next of the graph, usually the very end.
		LONGLONG pos;
		LIF(pMS->GetPositions(NULL, &pos)) ;
		LIF(pMS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning)) ;
	}
	// Release a interface
	SAFE_RELEASE(pMS) ;

	return hr ;
}

//------------------------------------------------------------------------------
// Name: CSampleApp::OnVolume/Balance()
// Desc: This method manipulate Volume/Balance
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::OnVolume(bool bUp)
{
	if (m_pGB == NULL)
		return E_ABORT;

	HRESULT hr ;

	// Query audio interface
	IBasicAudio * pBA;
	JIF(m_pGB->QueryInterface(IID_IBasicAudio, (void **)&pBA)) ;
	if (!pBA)
		return E_FAIL ;

	//----------------------------------
	//	fPos Range: (0 ~ 1)
	//	lfVolume Range: (-10,000 ~ 0)
	//----------------------------------
	float& fPos = m_fVolumePos ;
	double lfVolume ;

	fPos += (bUp ? 0.1f : -0.1f) ;

	// Saturation
	if (fPos < 0)
		fPos = 0 ;
	else if (fPos > 1)
		fPos = 1 ;
				
	// ܼ ó 
	// lfVolume = (fPos - 1) * 10000 ;

	// Scaling (fPos => lfVolume)
	double lfPower10 = pow(10, -10) ;	// 10 ^ (-10)
	lfVolume = (1 - lfPower10) * fPos + lfPower10 ;

	lfVolume = 10 * log10(lfVolume) ;	// decibel (-100dB ~ 0dB)
	lfVolume *= 100 ;					// DirectShow resolution (-10,000 ~ 0)

	// Set volume
	JIF(pBA->put_Volume((long)lfVolume)) ;
	SendMessage(m_hWnd, TBM_SETPOS, FALSE, 0); 

	// Release a interface
	SAFE_RELEASE(pBA);

	return hr ;
}


HRESULT CJYPlayerApp::OnBalance(bool bRight)
{
	if (m_pGB == NULL)
		return E_ABORT ;

	HRESULT hr ;

	// Query audio interface
	IBasicAudio * pBA;
	JIF(m_pGB->QueryInterface(IID_IBasicAudio, (void **)&pBA)) ;
	if (!pBA)
		return E_FAIL ;

	//----------------------------------
	//	fPos Range: (0 ~ 1)
	//	lfBalance Range: (-10,000 ~ 0)
	//----------------------------------
	float& fPos = m_fVolumePos ;
	double lfBalance ;

	fPos += (bRight ? 0.1f : -0.1f) ;

	// Saturation
	if (fPos < 0)
		fPos = 0 ;
	else if (fPos > 1)
		fPos = 1 ;

	// Position
	float fRightPos = fPos;
	float fLeftPos = 1 - fPos;

	// decibel values
	double lfRightdB, lfLeftdB;
	double lfPower10 = pow(10, -10) ;	// 10 ^ (-10)

	// Right dB
	lfRightdB = (1 - lfPower10) * fRightPos + lfPower10 ;	// Scaling
	lfRightdB = 10 * log10(lfRightdB) ;	// decibel (-100dB ~ 0dB)

	// Left dB
	lfLeftdB = (1 - lfPower10) * fLeftPos + lfPower10 ;	// Scaling
	lfLeftdB = 10 * log10(lfLeftdB) ;	// decibel (-100dB ~ 0dB)

	// Total value
	lfBalance = lfRightdB - lfLeftdB;	// decibel (-100dB ~ 100dB)
	lfBalance *= 100 ;					// DirectShow resolution (-10,000 ~ 10,000)

	// Set balance
	JIF(pBA->put_Balance((long)lfBalance)) ;

	// Release a interface
	SAFE_RELEASE(pBA);

	return hr;
}

//------------------------------------------------------------------------------
// Name: CJYPlayerApp::OnSize()
// Desc: This method manipulate WM_SIZE events
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::OnSize()
{
	if (m_pGB == NULL)
		return E_ABORT ;

	HRESULT hr ;

	IVideoWindow * pVW;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW));
	if (pVW != NULL)
	{
		// Track the movement of the container window and resize as needed
		RECT rcToolbar;
		GetWindowRect(m_hwndToolBar, &rcToolbar);
		int nToolbarHeight = rcToolbar.bottom - rcToolbar.top;

		RECT rect;
		GetClientRect(m_hWnd, &rect);
		LIF(pVW->SetWindowPosition(rect.left, rect.top + nToolbarHeight,
			rect.right - rect.left, rect.bottom -rect.top - nToolbarHeight));
	}
	// Release a interface
	SAFE_RELEASE(pVW);

	return hr;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::XXXFullScreen()
// Desc: This method will enter and exit fullscreen mode depending on the bool 
//       value.  We don't use IVideoWindow->putFullScreenMode because
//       it won't handle the mouse correctly.
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::ToggleFullScreen()
{
    return false == m_bFullScreenOn ? StartFullScreen() : StopFullScreen() ;
}

HRESULT CJYPlayerApp::StartFullScreen()
{
	if (m_pGB == NULL)
		return E_ABORT ;

	HRESULT hr ;

	// Query for video interfaces
	IVideoWindow * pVW ;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW)) ;
	if (!pVW)
		return E_FAIL ;

	// store the original window position
	long lLeft, lTop, lWidth, lHeight ;
	JIF(pVW->GetWindowPosition(&lLeft, &lTop, &lWidth, &lHeight)) ;
	SetRect(&m_rectOrgVideo, lLeft, lTop, lLeft + lWidth, lTop + lHeight) ;

	// save the original window style
	JIF(pVW->get_WindowStyle(&m_lOrgStyle)) ;
	JIF(pVW->get_WindowStyleEx(&m_lOrgStyleEx)) ;

	// the window can't be a child while it is full screen
	// or it can't be stretched beyond the parent window's borders
	JIF(pVW->put_Owner(NULL)) ;
	JIF(pVW->put_MessageDrain((OAHWND)m_hWnd)) ;

	// modify the window's style
	// ... remove these styles
	JIF(pVW->put_WindowStyle(m_lOrgStyle & ~(WS_BORDER|WS_CAPTION|WS_THICKFRAME))) ;
	// ... remove these extended styles
	JIF(pVW->put_WindowStyleEx(m_lOrgStyleEx	\
		& ~(WS_EX_CLIENTEDGE|WS_EX_STATICEDGE|WS_EX_WINDOWEDGE|WS_EX_DLGMODALFRAME)	\
		| WS_EX_TOPMOST)) ;

	// stretch the window to the full size of the screen
	long lScrnWidth = GetSystemMetrics(SM_CXSCREEN) ;
	long lScrnHeight = GetSystemMetrics(SM_CYSCREEN) ;
	JIF(pVW->SetWindowPosition(0, 0, lScrnWidth, lScrnHeight)) ;

	::ShowCursor(FALSE) ;	// make sure to show mouse now
	m_bFullScreenOn = true;

	// Release a interface
	SAFE_RELEASE(pVW) ;

	return true ;
}

HRESULT CJYPlayerApp::StopFullScreen()
{
	if (m_pGB == NULL)
		return E_ABORT ;

	HRESULT hr ;

	// Query for video interfaces
	IVideoWindow * pVW ;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW)) ;
	if (!(pVW != NULL))
		return E_FAIL ;

	// Track the movement of the container window and resize as needed
	RECT rcToolbar;
	GetWindowRect(m_hwndToolBar, &rcToolbar);
	int nToolbarHeight = rcToolbar.bottom - rcToolbar.top ;

	// restore the window's position
	RECT rect;
	GetClientRect(m_hWnd, &rect);
	JIF(pVW->SetWindowPosition(0, nToolbarHeight, rect.right, rect.bottom-nToolbarHeight)) ;

	// make it a child window again
	JIF(pVW->put_Owner(reinterpret_cast<OAHWND>(m_hWnd))) ;

	// restore the window's styles
	// ... restore the styles
	JIF(pVW->put_WindowStyle(m_lOrgStyle)) ;
	// ... restore the extended styles
	JIF(pVW->put_WindowStyleEx(m_lOrgStyleEx)) ;

	::ShowCursor(TRUE) ;  // make sure to show mouse now
	m_bFullScreenOn = false;

	SetForegroundWindow(m_hWnd) ;	
	SetFocus(m_hWnd) ;

	// Release a interface
	SAFE_RELEASE(pVW) ;

	return true ;
}


//------------------------------------------------------------------------------
// Name: CJYPlayerApp::HandleGraphEvent()
// Desc: This method manipulate filter graph events.
//------------------------------------------------------------------------------

HRESULT CJYPlayerApp::HandleGraphEvent(LPARAM lParam)
{
	if (m_pGB == NULL)
		return E_ABORT ;

	HRESULT hr ;

	long lEvent, lParam1, lParam2;
	long lTimeOut = 0;

	IMediaEventEx * pME;
	JIF(m_pGB->QueryInterface(IID_IMediaEventEx, (void **)&pME));

	//   ׳ ϸ Release()  Ƿ Ѵ!
	while (SUCCEEDED(pME->GetEvent(&lEvent, &lParam1, &lParam2, lTimeOut)))
	{
		switch (lEvent)
		{
		case EC_USERABORT:
			OnStopClip();
			break;

		case EC_COMPLETE:
			break;

		// TODO: Place code here.
		}

		hr = pME->FreeEventParams(lEvent, lParam1, lParam2);
	}
	// Release a interface
	SAFE_RELEASE(pME);

	return hr;
}

void CJYPlayerApp::StaticSetTime(HWND time)
{
	char strTime[MAX_PATH];
	
	if(nows > 59)
	{
		if(nowm++ > 59)
		{
			nowh++;
			
			nowm = 0;
		}
		nows = 0;
	}

	long pos = SendMessage(FileSeekTrackBar, TBM_GETPOS, 0, 0);


	if(pos >= totaltime)
	{
		KillTimer(m_hWnd, 1);
		Die = FALSE;
	}

	wsprintf(strTime, "%d: %d: %d", nowh, nowm, nows);
	SetWindowText(time, strTime);
}

void CJYPlayerApp::OnTimer()
{
	FileTrackPos++;
	SendMessage(FileSeekTrackBar, TBM_SETPOS, TRUE, (long)FileTrackPos);

	nows++;
	StaticSetTime(NowTimer);
}

void CJYPlayerApp::MoveTrackBar(HWND TrackBar, WORD Msg)
{
	LONGLONG pos;

	if(TrackBar == FileSeekTrackBar)
	{
		pos = SendMessage(FileSeekTrackBar, TBM_GETPOS, 0, 0);
		FileTrackPos = pos;
		OnPreviousClip(pos);
		
		nowm= (long)pos / 60;
		nowh = nowm / 60;
		nows = (long)pos % 60;

		if(!Die)
		{
			SetTimer(m_hWnd, 1, 1000, 0);
		}
	}
	
	if(TrackBar == SoundSeekTrackBar)
	{
		pos = SendMessage(SoundSeekTrackBar, TBM_GETPOS, 0, 0);
		SoundTrackPos = pos;

		switch(Msg)
		{
		case TB_LINEUP:
			OnVolume(false);
			break;

		case TB_LINEDOWN:
			OnVolume(true);
		}
	}
}


void CJYPlayerApp::CaptureScreen()
{
    if (m_pSG == NULL)    return;

    HRESULT hr;
    CComQIPtr<ISampleGrabber> pSampleGrabber(m_pSG);
    if (pSampleGrabber == NULL)     return;

    long lBufferSize;
    hr = pSampleGrabber->GetCurrentBuffer(&lBufferSize, NULL);
    if (FAILED(hr))       return;

    BYTE* pBuffer = new BYTE[lBufferSize];
    if (!pBuffer)        return;

    hr = pSampleGrabber->GetCurrentBuffer(&lBufferSize, (long *)pBuffer);
    if (SUCCEEDED(hr))
    {
        AM_MEDIA_TYPE mt;
        pSampleGrabber->GetConnectedMediaType(&mt); 
        VIDEOINFOHEADER *pviHeader = (VIDEOINFOHEADER *)mt.pbFormat;
        SaveImage(pviHeader->bmiHeader, pBuffer);
    }
    delete pBuffer;
}

void CJYPlayerApp::SaveImage(BITMAPINFOHEADER &bmiHeader, BYTE *pDIB)
{
	
	char pszFileName[MAX_PATH] = {"C:\\Documents and Settings\\admin\\My Documents\\My Pictures\\"};
	++Imagetemp;
	
	wsprintf(pszFileName,"%spicture%d.bmp", pszFileName,Imagetemp);
 

	
	BITMAPFILEHEADER bmfHeader;

	// only 24bit color bitmap 
	bmfHeader.bfType		= DIB_HEADER_MARKER;
	bmfHeader.bfOffBits		= sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	bmfHeader.bfSize		= sizeof(bmfHeader) + sizeof(BITMAPINFOHEADER) + bmiHeader.biSizeImage;
	bmfHeader.bfReserved1	= 0;
	bmfHeader.bfReserved2	= 0;

	HANDLE fd = CreateFile(pszFileName, GENERIC_WRITE, 0, NULL,
								CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	DWORD dwLength;
	
	if (!WriteFile(fd, (LPSTR)&bmfHeader, sizeof(bmfHeader), &dwLength, NULL))
		return;

	if (!WriteFile(fd, (LPSTR)&bmiHeader, sizeof(BITMAPINFOHEADER), &dwLength, NULL))
		return;

	if (!WriteFile(fd, pDIB, bmiHeader.biSizeImage, &dwLength, NULL))
		return;

	MessageBox(0, "My Document My picture  ĸĵǾϴ.","˸", MB_OK);

	CloseHandle(fd);
}

HRESULT CJYPlayerApp::OnURLOpen()
{
	HRESULT hr ;
	
	// ͱ׷Ŵ Ͽ ͱ׷  !
	if (m_pGB != NULL)
	{
		JIF(OnFileClose()) ;
	}
	// URL ҷ
	// Open URL Dialog ...
	
	// Retrieve the name of the selected file
	WCHAR wFileName[MAX_URL_PATH] ;
	
	if (CRenderURLDlg::GetOpenURLName(m_hInstance, m_hWnd, m_szFileName, MAX_URL_PATH) == IDOK)
	{
#ifndef UNICODE
		MultiByteToWideChar(CP_ACP, 0, m_szFileName, -1, wFileName, MAX_URL_PATH) ;
#else
		lstrcpy(wFileName, m_szFileName);
#endif
	}
	else
		return E_ABORT ;
	
    // Clear open dialog remnants before calling RenderFile()
	SetWindowText(m_hWnd, m_szFileName) ;
    UpdateWindow(m_hWnd) ;
	
	// ͱ׷Ŵ  (  )
    // Get the interface for DirectShow's GraphBuilder
    JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGB)) ;
	
	
	// Render File ...
	// Have the graph construct its the appropriate graph automatically
	JIF(m_pGB->RenderFile(wFileName, NULL)) ;
	
	// Query for video interfaces
	IVideoWindow * pVW ;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW)) ;
	if (pVW != NULL)
	{
		LIF(pVW->put_Owner((OAHWND)m_hWnd)) ;
		LIF(pVW->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN)) ;
		
		// resizing
		LIF(OnSize());
	}
	// Release a interface
	SAFE_RELEASE(pVW) ;
		
	// Get the video size
	IBasicVideo * pBV ;
	JIF(m_pGB->QueryInterface(IID_IBasicVideo, (void **)&pBV)) ;
	if (pBV != NULL)
	{
		long lWidth, lHeight ;
		hr = pBV->GetVideoSize(&lWidth, &lHeight) ;
		if (SUCCEEDED(hr))
		{
			// Track the movement of the container window and resize as needed
			RECT rcToolbar ;
			GetWindowRect(m_hwndToolBar, &rcToolbar) ;
			int nToolbarHeight = rcToolbar.bottom - rcToolbar.top ;
			
			SetWindowPos(m_hWnd, HWND_NOTOPMOST, 0, 0,
				lWidth, lHeight + nToolbarHeight,
				SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE) ;
		}
	}
	// Release a interface
	SAFE_RELEASE(pBV) ;
	
	// Set the owner window to receive event notices
	IMediaEventEx * pME ;
	JIF(m_pGB->QueryInterface(IID_IMediaEventEx, (void **)&pME)) ;
	if (pME != NULL)
	{
		LIF(pME->SetNotifyWindow((OAHWND)m_hWnd, WM_GRAPHNOTIFY, 0)) ;
	}
	// Release a interface
	SAFE_RELEASE(pME) ;
	
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(m_pGB, &m_dwGraphRegister) ;
    if (FAILED(hr))
    {
        TRACE(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr) ;
        m_dwGraphRegister = 0 ;
    }
#endif
	
	IBaseFilter *pAsfAcmHandler,*pWmAudioDecoder;
	hr = m_pGB->FindFilterByName(L"ASF ACM Handler",&pAsfAcmHandler);
	hr = m_pGB->FindFilterByName(L"WMAudio Decoder DMO",&pWmAudioDecoder);
	
	/*CComPtr<IBaseFilter> InfTee;
	JIF(CoCreateInstance(CLSID_InfTee,NULL,CLSCTX_INPROC_SERVER,IID_IBaseFilter,(void**)&InfTee));
	LIF(m_pGB->AddFilter(InfTee,L"Infinite Pin Tee Filter"));*/
	
	CComPtr<IBaseFilter> pFilterWAVDest;
	LIF(CoCreateInstance(CLSID_L544_WAVDest, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pFilterWAVDest)) ;
	LIF(m_pGB->AddFilter(pFilterWAVDest, L"WAV Dest"));
	
	CComPtr<IBaseFilter> pFilterFileWriter;
	LIF(CoCreateInstance(CLSID_L544_FileWriter, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pFilterFileWriter)) ;
	LIF(m_pGB->AddFilter(pFilterFileWriter, L"File Writer"));
	
	//.
	IBaseFilter * AudioRenderer;
	hr = m_pGB->FindFilterByName(L"Default DirectSound Device",&AudioRenderer);
	
	m_pGB->RemoveFilter(AudioRenderer);
	AudioRenderer->Release();
	
	IPin *pOutPin = NULL;
	
	IPin *pInPin = NULL;
	
	hr = GetPin(pAsfAcmHandler, PINDIR_OUTPUT, 0, &pOutPin);    //  Է ̸ 翬 PINDIR_INPUT  ϸ 
	
	hr = pOutPin->ConnectedTo(&pInPin);
	
	hr = pOutPin->Disconnect();
	hr = pInPin->Disconnect();
	
	static int fnum=0;
	
	CComQIPtr<IFileSinkFilter2> pFW(pFilterFileWriter);
	if (pFW != NULL)
	{
		LIF(pFW->SetFileName(L"C:\\Documents and Settings\\admin\\My Documents\\abc.wav", NULL));
	}
	else
	{
		CComQIPtr<IFileSinkFilter> pFW(pFilterFileWriter);
		LIF(pFilterFileWriter->QueryInterface(IID_IFileSinkFilter, (void **)&pFW));
		if (pFW != NULL)
		{
			LIF(pFW->SetFileName(L"C:\\Documents and Settings\\admin\\My Documents\\abc.wav", NULL));
			
		}
	}
	
	hr = ConnectFilters(m_pGB,pAsfAcmHandler,pFilterWAVDest);
	hr = ConnectFilters(m_pGB,pFilterWAVDest,pFilterFileWriter);
	



	return S_OK ;
}

HBITMAP CJYPlayerApp::MakeHBIT(HDC hdc)
{
	HANDLE hFile;
	DWORD Filesize, dwRead;
	PBYTE pData;
	HBITMAP hBit;
	hFile = CreateFile("picture1.bmp", GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	Filesize = GetFileSize(hFile, NULL);
	pData = new BYTE[Filesize];
	ReadFile(hFile, pData, Filesize, &dwRead, NULL);
	
	CloseHandle(hFile);
	
	hBit = CreateDIBitmap(hdc, (BITMAPINFOHEADER*)(pData + sizeof(BITMAPINFOHEADER)), CBM_INIT,
	((BITMAPFILEHEADER*)pData)->bfOffBits + pData, (BITMAPINFO*)(pData + sizeof(BITMAPFILEHEADER)), DIB_RGB_COLORS);
	delete[] pData;

	return hBit;
}



void CJYPlayerApp::DrawBitmap(HDC hdc, int x, int y, int w, int h, HBITMAP hbit)
{
	HDC MemDC;
	HBITMAP OldBitmap;

	MemDC = CreateCompatibleDC(hdc);
	OldBitmap = (HBITMAP)SelectObject(MemDC, hbit);

	BitBlt(hdc, x, y, w, h, MemDC, x, y, SRCCOPY);
	SelectObject(MemDC, OldBitmap);
	DeleteDC(MemDC);
}



void CJYPlayerApp::CaptureView()
{
	DialogBox(GetModuleHandle(0), MAKEINTRESOURCE(IDD_CAPTURE), m_hWnd, DialogProc2);
}



BOOL CALLBACK CJYPlayerApp::DialogProc2(HWND hwndDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uiMsg)
	{
	case WM_INITDIALOG:
		{
			HDC hdc = GetDC(hwndDlg);
			HBITMAP hbit = MakeHBIT(hdc);
			DrawBitmap(hdc, 50, 50, 440, 400, hbit);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
	
		case IDCANCEL:
			EndDialog(hwndDlg, IDCANCEL);
			return TRUE;
		}
	}

	return FALSE;
}

HRESULT CJYPlayerApp::OnFileOpenBit()
{
	HRESULT hr ;
	
	// ͱ׷Ŵ Ͽ ͱ׷  !
	if (m_pGB != NULL)
	{
		JIF(OnFileClose()) ;
	}
	
	// ̵  ҷ
	// Open File Dialog ...
    static OPENFILENAME ofn={0} ;
    static BOOL bSetInitialDir = FALSE ;
	
    ofn.lStructSize       = sizeof(OPENFILENAME) ;
    ofn.hwndOwner         = m_hWnd ;
    ofn.lpstrFilter       = FILTER_TEXT;
    ofn.lpstrCustomFilter = NULL ;
    ofn.nFilterIndex      = 1 ;
    ofn.lpstrFile         = m_szFileName ;
    ofn.nMaxFile          = MAX_PATH;
    ofn.lpstrTitle        = TEXT("Capture  ÿ...\0") ;
	ofn.lpstrFilter = "Bitmap File(*.bmp)\0*.bmp\0";
    ofn.lpstrFileTitle    = FileTitle;
	ofn.nMaxFileTitle	  = MAX_PATH;
    ofn.lpstrDefExt       = TEXT("*\0") ;
    ofn.Flags             = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST ;
	
    // Remember the path of the first selected file
    if (bSetInitialDir == FALSE)
    {
        ofn.lpstrInitialDir = DEFAULT_MEDIA_PATH ;
        bSetInitialDir = TRUE ;
    }
    else 
        ofn.lpstrInitialDir = NULL ;
	
	return S_OK ;
}

HRESULT CJYPlayerApp::OnFilter()
{
	HRESULT hr ;

	// ͱ׷Ŵ Ͽ ͱ׷  !
	if (m_pGB != NULL)
	{
		JIF(OnFileClose()) ;
	}

	// ̵  ҷ
	// Open File Dialog ...
    static OPENFILENAME ofn={0} ;
    static BOOL bSetInitialDir = FALSE ;

    ofn.lStructSize       = sizeof(OPENFILENAME) ;
    ofn.hwndOwner         = m_hWnd ;
    ofn.lpstrFilter       = FILE_FILTER_TEXT ;
    ofn.lpstrCustomFilter = NULL ;
    ofn.nFilterIndex      = 1 ;
    ofn.lpstrFile         = m_szFileName ;
    ofn.nMaxFile          = MAX_PATH ;
    ofn.lpstrTitle        = TEXT("Open Media File...\0") ;
    ofn.lpstrFileTitle    = FileTitle ;
	ofn.nMaxFileTitle	  = MAX_PATH;
    ofn.lpstrDefExt       = TEXT("*\0") ;
    ofn.Flags             = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST ;

    // Remember the path of the first selected file
    if (bSetInitialDir == FALSE)
    {
        ofn.lpstrInitialDir = DEFAULT_MEDIA_PATH ;
        bSetInitialDir = TRUE ;
    }
    else 
        ofn.lpstrInitialDir = NULL ;

	// Retrieve the name of the selected file
	WCHAR wFileName[MAX_PATH] ;

	if (GetOpenFileName((LPOPENFILENAME)&ofn))
	{
#ifndef UNICODE
		MultiByteToWideChar(CP_ACP, 0, m_szFileName, -1, wFileName, MAX_PATH) ;
#else
		lstrcpy(wFileName, m_szFileName);
#endif
	}
	else
		return E_ABORT ;

    // Clear open dialog remnants before calling RenderFile()
	SetWindowText(m_hWnd, m_szFileName) ;
    UpdateWindow(m_hWnd) ;

    JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGB)) ;



// --------------------------ĸ ----------------------------
	LIF(CoCreateInstance(CLSID_L544_SampleGrabber, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&m_pSG)) ;
	LIF(m_pGB->AddFilter(m_pSG, L"SampleGrabber"));
	
	AM_MEDIA_TYPE mt;
	ZeroMemory(&mt, sizeof(mt));
	mt.majortype = MEDIATYPE_Video;
	mt.subtype = MEDIASUBTYPE_RGB24;
	
	CComQIPtr<ISampleGrabber> pSampleGrabber(m_pSG);
	if (pSampleGrabber != NULL)
	{
		LIF(pSampleGrabber->SetMediaType(&mt));
		LIF(pSampleGrabber->SetBufferSamples(TRUE));
	}


	
    CComPtr<IUnknown> pUnk;	
    HRESULT hr1;

    CComPtr<IBaseFilter> pDynamicFilter;
    pDynamicFilter = (IBaseFilter *)new CInplaceTransformFilter(pUnk, &hr1);
    if (pDynamicFilter)
  	LIF(m_pGB->AddFilter(pDynamicFilter, L"Dynamic Filter"));


	// Render File ...
	// Have the graph construct its the appropriate graph automatically
	JIF(m_pGB->RenderFile(wFileName, NULL)) ;

	// Query for video interfaces
	IVideoWindow * pVW ;
	JIF(m_pGB->QueryInterface(IID_IVideoWindow, (void **)&pVW)) ;
	if (pVW != NULL)
	{
		LIF(pVW->put_Owner((OAHWND)m_hWnd)) ;
		LIF(pVW->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN)) ;

		// resizing
		LIF(OnSize());
	}
	// Release a interface
	
	SAFE_RELEASE(pVW) ;


	// Get the video size
	IBasicVideo * pBV ;
	JIF(m_pGB->QueryInterface(IID_IBasicVideo, (void **)&pBV)) ;
	if (pBV != NULL)
	{
		long lWidth, lHeight ;
		hr = pBV->GetVideoSize(&lWidth, &lHeight) ;
		if (SUCCEEDED(hr))
		{
			// Track the movement of the container window and resize as needed
			RECT rcToolbar ;
			GetWindowRect(m_hwndToolBar, &rcToolbar) ;
			int nToolbarHeight = rcToolbar.bottom - rcToolbar.top ;

			SetWindowPos(m_hWnd, HWND_NOTOPMOST, 0, 0,
				lWidth, lHeight + nToolbarHeight,
				SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE) ;
		}
	}
	// Release a interface
	SAFE_RELEASE(pBV) ;


	IMediaSeeking * pVWs;
		
	JIF(m_pGB->QueryInterface(IID_IMediaSeeking, (void **)&pVWs)) ;
	if (pVWs != NULL)
	{
		LONGLONG Duration;

		pVWs->GetDuration(&Duration);
		totaltime = Duration / ONE_SECOND;
		
		SendMessage(FileSeekTrackBar, TBM_SETRANGE, FALSE, MAKELPARAM(0, totaltime));
		
		nowm= (long)(Duration / ONE_SECOND) / 60;
		nowh = nowm / 60;
		nows = (long)(Duration / ONE_SECOND) % 60;
		StaticSetTime(TotalTimer);

	}




#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(m_pGB, &m_dwGraphRegister) ;
    if (FAILED(hr))
    {
        TRACE(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr) ;
        m_dwGraphRegister = 0 ;
    }
#endif
		return S_OK ;

}



HRESULT CJYPlayerApp::GetPin( IBaseFilter * pFilter, PIN_DIRECTION dirrequired, int iNum, IPin **ppPin)
{
    IEnumPins * pEnum = NULL;
	
    *ppPin = NULL;
	
    if (!pFilter)
		
		return E_POINTER;
	
    HRESULT hr = pFilter->EnumPins(&pEnum);
	
    if(FAILED(hr)) 
		return hr;

    ULONG ulFound;
	
    IPin *pPin;
	
    hr = E_FAIL;
    pEnum->Reset();
    while(S_OK == pEnum->Next(1, &pPin, &ulFound))	
    {	
        PIN_DIRECTION pindir = (PIN_DIRECTION)3;		
        pPin->QueryDirection(&pindir);	
        if(pindir == dirrequired)		
        {		
            if(iNum == 0)			
            {		
                *ppPin = pPin;  // Return the pin's interface		
                hr = S_OK;      // Found requested pin, so clear error		
                break;		
            }	
            iNum--;	
        } 	
        pPin->Release();	
    } 
    pEnum->Release();
    return hr;
}