//------------------------------------------------------------------------------
// File: fInplaceTransform.cpp
//
// Desc: implement CInplaceTransformFilter class
//
// Copyright (c) 2000 - 2005, L544 Technology.  All rights reserved.
//------------------------------------------------------------------------------

#include "stdafx.h"
#include "fInplaceTransform.h"
#include "fInplaceTransform_ip.h"
#include "fInplaceTransform_op.h"

//
// Constructor
//
CInplaceTransformFilter::CInplaceTransformFilter(LPUNKNOWN pUnk, HRESULT *phr)
	: CTransInPlaceFilter(NAME("CInplaceTransformFilter"), pUnk, CLSID_NULL, phr)
{
    CAutoLock cAutoLock(&m_csFilter);

	if (SUCCEEDED(*phr))
	{
		// Create an output pin
		CInplaceTransformOutputPin *pOut = new CInplaceTransformOutputPin(phr, this, L"Out");
		if (pOut)
		{
			if (SUCCEEDED(*phr))
				m_pOutput = pOut;
			else
				delete pOut;
		}
		else
			*phr = E_OUTOFMEMORY;

		//
		// NOTE!: If we've created our own output pin we must also create
		// the input pin ourselves because the CTransformFilter base class 
		// will create an extra output pin if the input pin wasn't created.        
		//
		CInplaceTransformInputPin *pIn = new CInplaceTransformInputPin(phr, this, L"In");
		if (pIn)
		{
			if (SUCCEEDED(*phr))
				m_pInput = pIn;
			else
				delete pIn;
		}
		else
			*phr = E_OUTOFMEMORY;
	}
}


//
// Destructor
//
CInplaceTransformFilter::~CInplaceTransformFilter()
{
	// CTransformFilter::~CTranformFilter() deletes two pins.
}


//
// Transform
//
HRESULT CInplaceTransformFilter::Transform(IMediaSample *pSample)
{
	// get graphic infomation
	AM_MEDIA_TYPE* pMediaType = &m_pInput->CurrentMediaType();
	VIDEOINFO *pvi = (VIDEOINFO *)pMediaType->pbFormat;

	int nWidth = pvi->bmiHeader.biWidth;
	int nHeight = pvi->bmiHeader.biHeight;
	int nBitCount = pvi->bmiHeader.biBitCount;
	int nScanLine = (((nWidth * nBitCount) + 31) & ~31) / 8;

	// get data pointers
	BYTE *pData;
	HRESULT hr;

	hr = pSample->GetPointer(&pData);
	if (FAILED(hr))
		return hr;

	// horizontal-flip routine
	// just transform data (in-place), other data remains.
	for (int i=0; i<nHeight/2; i++)
	{
		CopyMemory(&pData[i*nScanLine], &pData[(nHeight-i-1)*nScanLine], nScanLine);
	}

	return S_OK;
}


//
// CheckInputType
//
HRESULT CInplaceTransformFilter::CheckInputType(const CMediaType *pmtIn)
{
	// only if we share the buffer after reconnect()
	if (m_pOutput->IsConnected())
	{
		if (*pmtIn == m_pOutput->CurrentMediaType())
			return S_OK;
		else
			return E_FAIL;
	}

	// connection time before reconnect()!
	// RGB format and VIDEOINFO are needed...
	if (IsEqualGUID(*pmtIn->Type(), MEDIATYPE_Video)) 
	{
		if (IsEqualGUID(*pmtIn->Subtype(), MEDIASUBTYPE_RGB24) ||
			IsEqualGUID(*pmtIn->Subtype(), MEDIASUBTYPE_RGB32) ||
			IsEqualGUID(*pmtIn->Subtype(), MEDIASUBTYPE_RGB555) ||
			IsEqualGUID(*pmtIn->Subtype(), MEDIASUBTYPE_RGB565) ||
			IsEqualGUID(*pmtIn->Subtype(), MEDIASUBTYPE_RGB8)
			)
		{
			if (IsEqualGUID(*pmtIn->FormatType(), FORMAT_VideoInfo) == FALSE)
				return E_INVALIDARG;
			else
				return S_OK;
		}
	}

	return E_FAIL;
}
